package loja;

import java.math.BigDecimal;

public class ItemVO  implements java.io.Serializable {
	
	private String numero;
	private String compraId;
	private String produtoId;
	private ProdutoVO produto; // pode ser null se usada verso light
	private int quantidade;
	
	public ItemVO() {}
	
	/**
	 * Verso contendo apenas cdigo do produto. Produto  null.
	 */
	public ItemVO(String numero, String produtoId, String compraId, int quantidade) {
		this.numero = numero;
		this.compraId = compraId;
		this.produtoId = produtoId;
		this.quantidade = quantidade;
	}
	
	/** 
	 * Versao usada pelo carrinho de compras 
	 */
	public ItemVO(ProdutoVO produto, int quantidade) {
		this.numero = produto.getCodigo();
		this.produto = produto;
		this.produtoId = produto.getCodigo();
		this.quantidade = quantidade;
	}
	
	/**
	 * Versao completa
	 */
	public ItemVO(String numero, ProdutoVO produto, String compraId, int quantidade) {
		this.numero = numero;
		this.compraId = compraId;
		this.produto = produto;
		this.produtoId = produto.getCodigo();
		this.quantidade = quantidade;
	}
	
	public void setNumero(String numero) {
		this.numero = numero;
	}
	public String getNumero() {
		return numero;
	}

	// Pode ser null quando usado em carrinho de compras
	public String getCompraId() {
		return compraId;
	}
	public void setCompraId(String compraId) {
		this.compraId = compraId;
	}
	
	public void setProdutoId(String produtoId) {
		this.produtoId = produtoId;
	}
	public String getProdutoId() {
		return produtoId;
	}
	
	public void setProduto(ProdutoVO produto) {
		this.produto = produto;
		this.produtoId = produto.getCodigo();
	}
	public ProdutoVO getProduto() {
		return produto;
	}
	
	public void setQuantidade(int quantidade) {
		this.quantidade = quantidade;
	}
	public int getQuantidade() {
		return quantidade;
	}
	
	// Metodos abaixo so funcionam se objeto for criado com produto completo (e nao apenas codigo)
	/**
	 * So funciona na verso completa.
	 */
	public double getSubTotal() {
		if (produto != null) {
			return produto.getPreco() * quantidade;
		}
		return -99.99;
	}
	/**
	 * So funciona na verso completa.
	 */
	public String getNomeProduto() {
		return produto.getNome();
	}
	/**
	 * So funciona na verso completa.
	 */
	public String getCodProduto() {
		return produto.getCodigo();
	}
	/**
	 * So funciona na verso completa.
	 */
	public String getPrecoProduto() {
		return "" + produto.getPreco();
	}
	
	//////////////////
	public int hashCode() {
		return Integer.parseInt(numero) * 37;
	}
	
	public boolean equals(Object o) {
		if (!(o instanceof ItemVO)) return false;
		ItemVO p = (ItemVO) o;
		return p.numero.equals(numero);
	}
	
	public String toString() {
		return "["+numero + "] Produto: " + produtoId + ", Quantidade: " + quantidade + ", $" + getSubTotal();
	}

}
