package loja.integration;

import loja.*;

import java.sql.*;
import java.util.*;
import java.math.BigDecimal;

/**
create table produtos (
	id integer constraint pk_produtos primary key, 
	nome varchar(64), 
	preco numeric(10,2),
	qte   integer
);
*/
public class JdbcProdutoDAO implements ProdutoDAO {
	
	/** SQL queries */
	public static final String SQL_CODIGO = "select max(id) from produtos";
	public static final String SQL_CREATE = "insert into produtos(id, nome, preco, qte) values (?, ?, ?, ?)";
	public static final String SQL_REMOVE = "delete from produtos where id = ?";
	public static final String SQL_LOAD   = "select id, nome, preco, qte from produtos where id = ?";
	public static final String SQL_STORE  = "update produtos set nome = ?, preco = ?, qte = ? where id = ?";
	public static final String SQL_PK     = "select id from produtos where id = ?";
	public static final String SQL_ALL    = "select id from produtos";
	
	private static ProdutoDAO instance = null;
	private ServiceLocator locator;
	
	private JdbcProdutoDAO() throws LojaException {
		locator = ServiceLocator.getInstance();
	}
	
	private Connection getConnection() throws LojaException {
		Connection con = locator.findJdbcConnection("jdbc/LojaDB");
		return con;
	}
	
	public static ProdutoDAO getInstance()  throws LojaException {
		if (instance == null) {
			instance = new JdbcProdutoDAO();
		}
		return instance;
	}
	
	public String criarCodigo() throws LojaException {
		String sql = SQL_CODIGO;
		System.out.println("Executando..." + sql);
		Connection con = null;
		PreparedStatement stmt = null;
		String codigo = null;
		try {
			con = getConnection();
			stmt = con.prepareStatement(sql);
			stmt.clearParameters();
			ResultSet rs = stmt.executeQuery();
			if (rs.next()) {
				int ultimo = rs.getInt(1);
				codigo = "" + (ultimo + 1);
			} else {
				codigo = "1";
			}
			if (codigo == null) {
				throw new SQLException("Incapaz de criar novo codigo!");
			}
		} catch (SQLException e) {
			throw new LojaException(e);
		} finally {
			try {	
				stmt.close();
				con.close(); 
			} catch (SQLException e) {}
		}
		return codigo;
	}
	
	public String create(ProdutoVO produto) throws LojaException {
		String sql = SQL_CREATE;
		System.out.println("Executando..." + sql);
		Connection con = null;
		PreparedStatement stmt = null;
		String codigo = null;
		try {
			con = getConnection();
			stmt = con.prepareStatement(sql);
			stmt.clearParameters();
			codigo = produto.getCodigo();
			stmt.setInt(1, Integer.parseInt(codigo));
			stmt.setString(2, produto.getNome());
			stmt.setDouble(3, produto.getPreco());
			stmt.setInt(4, produto.getQuantidade());
			stmt.executeUpdate();
		} catch (SQLException e) {
			throw new LojaException(e);
		} finally {
			try {	
				stmt.close();
				con.close(); 
			} catch (SQLException e) {}
		}
		return codigo;
	}
	
	public void remove(String codigo) throws LojaException {
		String sql = SQL_REMOVE;
		System.out.println("Executando..." + sql);
		Connection con = null;
		PreparedStatement stmt = null;
		try {
			con = getConnection();
			stmt = con.prepareStatement(sql);
			stmt.clearParameters();
			stmt.setInt(1, Integer.parseInt(codigo));
			stmt.executeUpdate();
		} catch (SQLException e) {
			throw new LojaException(e);
		} finally {
			try {	
				stmt.close();
				con.close(); 
			} catch (SQLException e) {}
		}
	}
	
	public ProdutoVO load(String codigo) throws LojaException  {
		String sql = SQL_LOAD;
		System.out.println("Executando..." + sql);
		Connection con = null;
		PreparedStatement stmt = null;
		ProdutoVO produto = new ProdutoVO();
		try {
			con = getConnection();
			stmt = con.prepareStatement(sql);
			stmt.clearParameters();
			stmt.setInt(1, Integer.parseInt(codigo));
			ResultSet rs = stmt.executeQuery();
			if (rs.next()) {
				produto.setCodigo(""+rs.getInt(1));
				produto.setNome(rs.getString(2));
				produto.setPreco(rs.getDouble(3));
				produto.setQuantidade(rs.getInt(4));
			}
		} catch (SQLException e) {
			throw new LojaException(e);
		} finally {
			try {	
				stmt.close();
				con.close(); 
			} catch (SQLException e) {}
		}
		return produto;
	}
	
	public void store(ProdutoVO produto) throws LojaException {
		String sql = SQL_STORE;
		System.out.println("Executando..." + sql);
		Connection con = null;
		PreparedStatement stmt = null;
		try {
			con = getConnection();
			stmt = con.prepareStatement(sql);
			stmt.clearParameters();
			stmt.setString(1, produto.getNome());
			stmt.setDouble(2, produto.getPreco());
			stmt.setInt(3, produto.getQuantidade());
			stmt.setInt(4, Integer.parseInt(produto.getCodigo()));
			stmt.executeUpdate();
		} catch (SQLException e) {
			throw new LojaException(e);
		} finally {
			try {	
				stmt.close();
				con.close(); 
			} catch (SQLException e) {}
		}
	}
	
	public boolean exists(String codigo) throws LojaException {
		String sql = SQL_PK;
		System.out.println("Executando..." + sql);
		Connection con = null;
		PreparedStatement stmt = null;
		int codigoLido = -1;
		try {
			con = getConnection();
			stmt = con.prepareStatement(sql);
			stmt.clearParameters();
			stmt.setInt(1, Integer.parseInt(codigo));
			ResultSet rs = stmt.executeQuery();
			if (rs.next()) {
				codigoLido = rs.getInt(1);
			}
			if (codigoLido == -1) {
				throw new SQLException("Registro " + codigo + " inexistente!");
			}
			return true;
		} catch (SQLException e) {
			throw new LojaException(e);
		} finally {
			try {	
				stmt.close();
				con.close(); 
			} catch (SQLException e) { return false; }
		}
	}
	
	public String[] getAll() throws LojaException {
		String sql = SQL_ALL;
		System.out.println("Executando " + sql);
		Connection con = null;
		PreparedStatement stmt = null;
		List codigoList = new ArrayList();
		try {
			con = getConnection();
			stmt = con.prepareStatement(sql);
			stmt.clearParameters();
			ResultSet rs = stmt.executeQuery();
			while (rs.next()) {
				codigoList.add(""+rs.getInt(1));
			}
			String[] buffer = new String[codigoList.size()];
			return (String[]) codigoList.toArray(buffer);
		} catch (SQLException e) {
			throw new LojaException(e);
		} finally {
			try {	
				stmt.close();
				con.close(); 
			} catch (SQLException e) {}
		}
	}
}
